package zero1hd.rhythmbullet.audio;

import java.io.IOException;

import org.jaudiotagger.audio.exceptions.InvalidAudioFrameException;
import org.jaudiotagger.audio.exceptions.ReadOnlyFileException;
import org.jaudiotagger.audio.mp3.MP3File;
import org.jaudiotagger.tag.TagException;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.files.FileHandle;

import javazoom.jl.decoder.Bitstream;
import javazoom.jl.decoder.BitstreamException;
import javazoom.jl.decoder.DecoderException;
import javazoom.jl.decoder.MP3Decoder;
import javazoom.jl.decoder.OutputBuffer;

public class Mp3AudioData implements AudioData {
	private int readWindowSize = 1024;

	private Music playbackMusic;
	private int readIndex;
	
	private int sampleRate;
	private int sampleCount;
	private float durationInSeconds;
	
	Bitstream bitStream;
	MP3Decoder decoder;
	public Mp3AudioData(FileHandle audioFile) {
		try {
			MP3File mp3File = new MP3File(audioFile.file());
			
			sampleCount = (int) mp3File.getMP3AudioHeader().getNumberOfFrames();
			durationInSeconds = mp3File.getMP3AudioHeader().getNumberOfFrames()/readWindowSize;
			sampleRate = mp3File.getMP3AudioHeader().getSampleRateAsNumber();
			
			bitStream = new Bitstream(audioFile.read());
			decoder = new MP3Decoder();
		} catch (IOException | TagException | ReadOnlyFileException | InvalidAudioFrameException e1) {
			e1.printStackTrace();
		}
		reset();

		playbackMusic = Gdx.audio.newMusic(audioFile);
	}

	@Override
	public void readIndexUpdate() {
		readIndex = (int) (playbackMusic.getPosition() * sampleRate / readWindowSize);
	}

	@Override
	public int getReadIndex() {
		return readIndex;
	}

	@Override
	public void reset() {
		playbackMusic.stop();
	}

	@Override
	public int getReadWindowSize() {
		return readWindowSize;
	}

	@Override
	public Music getPlaybackMusic() {
		return playbackMusic;
	}

	@Override
	public int getSampleCount() {
		return sampleCount;
	}

	@Override
	public void dispose() {
		reset();
		playbackMusic.dispose();
	}

	@Override
	public float getDuration() {
		return durationInSeconds;
	}

	@Override
	public int readSamples(float[] samples) {
		int samplesRead = 0;
		try {
			OutputBuffer samplesBuffer = decoder.decodeFrame(bitStream.readFrame(), bitStream);
			byte[] frameBuffer = samplesBuffer.getBuffer();
			for (int sampleID = 0; sampleID < samples.length; sampleID++) {
				
				samples[sampleID] += (short) ((frameBuffer[1] << 8) + (frameBuffer[0] & 0xff));
				if (samplesBuffer.isStereo()) {
					samples[sampleID] = (short) ((frameBuffer[3] << 8) + (frameBuffer[2] & 0xff));
					
					samples[sampleID] /= 2;
				}
				
				samples[sampleID] /= Short.MAX_VALUE+1;
				samplesRead ++;
			}
		} catch (DecoderException | BitstreamException e) {
			e.printStackTrace();
		}
		return samplesRead;
	}

	@Override
	public float getSampleRate() {
		return sampleRate;
	}

}
